
/*
 * Program to test 9-Bit Single Track Gray Encoder
 * Decoder: https://www.thingiverse.com/thing:4590077
 * Date: September 2020 
 * Atuhor: atzensepp  Florian Bauer
 * 
 * Description:
 *   This program reads the phototransistors of the encoder and translates it to angle values
 *   It has a opotional interrupt driven Multiplexing-Routine for a 3-Digit 7-Segment -Display to display the angle in degrees
 *   
 * Hardware:
 * 9 LEDs (inner Ring of Decoder):
 * VCC --- [470 Ohm ]--- (LED) >| ---- GND  
 * 
 * 9 Photo Transistors (PT) (Sensor Blocks of Decoder in outer Ring):
 * VCC ---- [ 47k Ohm] --- Collector of PT and Arduino-Pin
 * GND ------------------- Emitter of PT                      
 *   Light:    0.24V
 *   No Light:  5 V
 *   
 * Arduino 
 * 
 * Pin 2: Track 1
 * Pin 3: Track 2
 * ...
 * Pin 10: Track 9 
 * 
 * Multipelxed 3-Digit 7-Segment-Display
 * 3 x 7-Segment-Display (Common Anode)
 * Anodes via npn-Transistors T1,T2,T3 to VCC
 * T1.Collector, T2.Collector, T3.Collector - VCC
 * T1.Emitter   -  7-Seg Display for Digit  100 Pin 3,8 / Common Anode 
 * T2.Emitter   -  7-Seg Display for Digit   10 Pin 3,8 / Common Anode
 * T3.Emitter   -  7-Seg Display for Digit    1 Pin 3,8 / Common Anode
 * 
 * Segment-Cathodes a-f via 100 Ohm Resistors to corresponding outputs of 7-Segment-Driver 7447
 * 7447
 * Pin 7 (A)    - 74HC595 Pin 15
 * Pin 1 (B)    - 74HC595 Pin  1 
 * Pin 2 (C)    - 74HC595 Pin  2
 * Pin 6 (D)    - 74HC595 Pin  3
 * Pin 3 (LT)   - VCC
 * Pin 16       - VCC 
 * Pin 8        - GND
 * Pin 13 (a)   - [ 100 Ohm ] - 7-Seg Display Pin 7
 * Pin 12 (b)   - [ 100 Ohm ] - 7-Seg Display Pin 6
 * Pin 11 (c)   - [ 100 Ohm ] - 7-Seg Display Pin 4
 * Pin 10 (d)   - [ 100 Ohm ] - 7-Seg Display Pin 2
 * Pin 9  (e)   - [ 100 Ohm ] - 7-Seg Display Pin 1
 * Pin 15 (f)   - [ 100 Ohm ] - 7-Seg Display Pin 9
 * Pin 14 (g)   - [ 100 Ohm ] - 7-Seg Display Pin 10
 * 
 * 
 * 74HC595 Shift Register:
 * Pin 16 (VCC)  - VCC / 5V
 * Pin 15 (A)    - 7447 Pin 7
 * Pin 1  (B)    - 7447 Pin 1
 * Pin 2  (C)    - 7447 Pin 2
 * Pin 3  (D)    - 7447 Pin 6
 * Pin 4  (E)    - [ 1k ]- T1 .Base
 * Pin 5  (F)    - [ 1k ]- T2 .Base
 * Pin 6  (G)    - [ 1k ]- T3 .Base
 * Pin 8  (GND)  - GND
 * Pin 10 (CLR)      - VCC
 * Pin 11 (CRCK)     -  Arduino Pin 17
 * Pin 12 (LATCH CK) - Arduino Pin 15
 * Pin 13 (EN)       - GND
 * Pin 14 (DATA)     - Arduino Pin 14
 * 
 * 
 */
 
// Comment out if no Display not requierd:
#define DISPLAY_THREE_DIGITS
#define BITS6 
//#define BITS9

#ifdef DISPLAY_THREE_DIGITS
//Pin connected to Pin 12 (LATCH CK) of 74HC595
int latchPin = 15;
//Pin connected to Pin 11 (SRCK)  of 74HC595
int clockPin = 14;
////Pin connected to Pin 14 (DATA) of 74HC595
int dataPin = 17;

volatile  int display_value=0;
volatile int digit=0;
#endif

int angle[512];

void setup() {
  unsigned int i;
#ifdef BITS9
  angle[0b011000000]=0;
angle[0b011010000]=1;
angle[0b011010001]=2;
angle[0b011000001]=3;
angle[0b011000101]=4;
angle[0b010000101]=5;
angle[0b010100101]=6;
angle[0b010110101]=7;
angle[0b011110101]=8;
angle[0b011100101]=9;
angle[0b011101101]=10;
angle[0b111101101]=11;
angle[0b110101101]=12;
angle[0b110101100]=13;
angle[0b110001100]=14;
angle[0b110011100]=15;
angle[0b110010100]=16;
angle[0b110110100]=17;
angle[0b111110100]=18;
angle[0b111110000]=19;
angle[0b111110001]=20;
angle[0b111010001]=21;
angle[0b111000001]=22;
angle[0b111000101]=23;
angle[0b111001101]=24;
angle[0b111001111]=25;
angle[0b111011111]=26;
angle[0b111010111]=27;
angle[0b110010111]=28;
angle[0b010010111]=29;
angle[0b000010111]=30;
angle[0b000110111]=31;
angle[0b000100111]=32;
angle[0b000100101]=33;
angle[0b100100101]=34;
angle[0b101100101]=35;
angle[0b101100001]=36;
angle[0b001100001]=37;
angle[0b001000001]=38;
angle[0b001000000]=39;
angle[0b001100000]=40;
angle[0b001101000]=41;
angle[0b101101000]=42;
angle[0b101100000]=43;
angle[0b101100010]=44;
angle[0b101000010]=45;
angle[0b101010010]=46;
angle[0b101011010]=47;
angle[0b101111010]=48;
angle[0b101110010]=49;
angle[0b101110110]=50;
angle[0b111110110]=51;
angle[0b111010110]=52;
angle[0b011010110]=53;
angle[0b011000110]=54;
angle[0b011001110]=55;
angle[0b011001010]=56;
angle[0b011011010]=57;
angle[0b011111010]=58;
angle[0b011111000]=59;
angle[0b111111000]=60;
angle[0b111101000]=61;
angle[0b111100000]=62;
angle[0b111100010]=63;
angle[0b111100110]=64;
angle[0b111100111]=65;
angle[0b111101111]=66;
angle[0b111101011]=67;
angle[0b111001011]=68;
angle[0b101001011]=69;
angle[0b100001011]=70;
angle[0b100011011]=71;
angle[0b100010011]=72;
angle[0b100010010]=73;
angle[0b110010010]=74;
angle[0b110110010]=75;
angle[0b110110000]=76;
angle[0b100110000]=77;
angle[0b100100000]=78;
angle[0b000100000]=79;
angle[0b000110000]=80;
angle[0b000110100]=81;
angle[0b010110100]=82;
angle[0b010110000]=83;
angle[0b010110001]=84;
angle[0b010100001]=85;
angle[0b010101001]=86;
angle[0b010101101]=87;
angle[0b010111101]=88;
angle[0b010111001]=89;
angle[0b010111011]=90;
angle[0b011111011]=91;
angle[0b011101011]=92;
angle[0b001101011]=93;
angle[0b001100011]=94;
angle[0b001100111]=95;
angle[0b001100101]=96;
angle[0b001101101]=97;
angle[0b001111101]=98;
angle[0b001111100]=99;
angle[0b011111100]=100;
angle[0b011110100]=101;
angle[0b011110000]=102;
angle[0b011110001]=103;
angle[0b011110011]=104;
angle[0b111110011]=105;
angle[0b111110111]=106;
angle[0b111110101]=107;
angle[0b111100101]=108;
angle[0b110100101]=109;
angle[0b110000101]=110;
angle[0b110001101]=111;
angle[0b110001001]=112;
angle[0b010001001]=113;
angle[0b011001001]=114;
angle[0b011011001]=115;
angle[0b011011000]=116;
angle[0b010011000]=117;
angle[0b010010000]=118;
angle[0b000010000]=119;
angle[0b000011000]=120;
angle[0b000011010]=121;
angle[0b001011010]=122;
angle[0b001011000]=123;
angle[0b101011000]=124;
angle[0b101010000]=125;
angle[0b101010100]=126;
angle[0b101010110]=127;
angle[0b101011110]=128;
angle[0b101011100]=129;
angle[0b101011101]=130;
angle[0b101111101]=131;
angle[0b101110101]=132;
angle[0b100110101]=133;
angle[0b100110001]=134;
angle[0b100110011]=135;
angle[0b100110010]=136;
angle[0b100110110]=137;
angle[0b100111110]=138;
angle[0b000111110]=139;
angle[0b001111110]=140;
angle[0b001111010]=141;
angle[0b001111000]=142;
angle[0b101111000]=143;
angle[0b101111001]=144;
angle[0b111111001]=145;
angle[0b111111011]=146;
angle[0b111111010]=147;
angle[0b111110010]=148;
angle[0b111010010]=149;
angle[0b111000010]=150;
angle[0b111000110]=151;
angle[0b111000100]=152;
angle[0b101000100]=153;
angle[0b101100100]=154;
angle[0b101101100]=155;
angle[0b001101100]=156;
angle[0b001001100]=157;
angle[0b001001000]=158;
angle[0b000001000]=159;
angle[0b000001100]=160;
angle[0b000001101]=161;
angle[0b000101101]=162;
angle[0b000101100]=163;
angle[0b010101100]=164;
angle[0b010101000]=165;
angle[0b010101010]=166;
angle[0b010101011]=167;
angle[0b010101111]=168;
angle[0b010101110]=169;
angle[0b110101110]=170;
angle[0b110111110]=171;
angle[0b110111010]=172;
angle[0b110011010]=173;
angle[0b110011000]=174;
angle[0b110011001]=175;
angle[0b010011001]=176;
angle[0b010011011]=177;
angle[0b010011111]=178;
angle[0b000011111]=179;
angle[0b000111111]=180;
angle[0b000111101]=181;
angle[0b000111100]=182;
angle[0b010111100]=183;
angle[0b110111100]=184;
angle[0b111111100]=185;
angle[0b111111101]=186;
angle[0b011111101]=187;
angle[0b011111001]=188;
angle[0b011101001]=189;
angle[0b011100001]=190;
angle[0b011100011]=191;
angle[0b011100010]=192;
angle[0b010100010]=193;
angle[0b010110010]=194;
angle[0b010110110]=195;
angle[0b000110110]=196;
angle[0b000100110]=197;
angle[0b000100100]=198;
angle[0b000000100]=199;
angle[0b000000110]=200;
angle[0b100000110]=201;
angle[0b100010110]=202;
angle[0b000010110]=203;
angle[0b001010110]=204;
angle[0b001010100]=205;
angle[0b001010101]=206;
angle[0b101010101]=207;
angle[0b101010111]=208;
angle[0b001010111]=209;
angle[0b011010111]=210;
angle[0b011011111]=211;
angle[0b011011101]=212;
angle[0b011001101]=213;
angle[0b011001100]=214;
angle[0b111001100]=215;
angle[0b101001100]=216;
angle[0b101001101]=217;
angle[0b101001111]=218;
angle[0b100001111]=219;
angle[0b100011111]=220;
angle[0b100011110]=221;
angle[0b000011110]=222;
angle[0b001011110]=223;
angle[0b011011110]=224;
angle[0b011111110]=225;
angle[0b111111110]=226;
angle[0b101111110]=227;
angle[0b101111100]=228;
angle[0b101110100]=229;
angle[0b101110000]=230;
angle[0b101110001]=231;
angle[0b001110001]=232;
angle[0b001010001]=233;
angle[0b001011001]=234;
angle[0b001011011]=235;
angle[0b000011011]=236;
angle[0b000010011]=237;
angle[0b000010010]=238;
angle[0b000000010]=239;
angle[0b000000011]=240;
angle[0b010000011]=241;
angle[0b010001011]=242;
angle[0b000001011]=243;
angle[0b000101011]=244;
angle[0b000101010]=245;
angle[0b100101010]=246;
angle[0b110101010]=247;
angle[0b110101011]=248;
angle[0b100101011]=249;
angle[0b101101011]=250;
angle[0b101101111]=251;
angle[0b101101110]=252;
angle[0b101100110]=253;
angle[0b001100110]=254;
angle[0b011100110]=255;
angle[0b010100110]=256;
angle[0b110100110]=257;
angle[0b110100111]=258;
angle[0b110000111]=259;
angle[0b110001111]=260;
angle[0b010001111]=261;
angle[0b000001111]=262;
angle[0b000101111]=263;
angle[0b001101111]=264;
angle[0b001111111]=265;
angle[0b011111111]=266;
angle[0b010111111]=267;
angle[0b010111110]=268;
angle[0b010111010]=269;
angle[0b010111000]=270;
angle[0b110111000]=271;
angle[0b100111000]=272;
angle[0b100101000]=273;
angle[0b100101100]=274;
angle[0b100101101]=275;
angle[0b100001101]=276;
angle[0b100001001]=277;
angle[0b000001001]=278;
angle[0b000000001]=279;
angle[0b100000001]=280;
angle[0b101000001]=281;
angle[0b101000101]=282;
angle[0b100000101]=283;
angle[0b100010101]=284;
angle[0b000010101]=285;
angle[0b010010101]=286;
angle[0b011010101]=287;
angle[0b111010101]=288;
angle[0b110010101]=289;
angle[0b110110101]=290;
angle[0b110110111]=291;
angle[0b010110111]=292;
angle[0b010110011]=293;
angle[0b000110011]=294;
angle[0b001110011]=295;
angle[0b001010011]=296;
angle[0b011010011]=297;
angle[0b111010011]=298;
angle[0b111000011]=299;
angle[0b111000111]=300;
angle[0b101000111]=301;
angle[0b100000111]=302;
angle[0b100010111]=303;
angle[0b100110111]=304;
angle[0b100111111]=305;
angle[0b101111111]=306;
angle[0b101011111]=307;
angle[0b001011111]=308;
angle[0b001011101]=309;
angle[0b001011100]=310;
angle[0b011011100]=311;
angle[0b010011100]=312;
angle[0b010010100]=313;
angle[0b010010110]=314;
angle[0b110010110]=315;
angle[0b110000110]=316;
angle[0b110000100]=317;
angle[0b100000100]=318;
angle[0b100000000]=319;
angle[0b110000000]=320;
angle[0b110100000]=321;
angle[0b110100010]=322;
angle[0b110000010]=323;
angle[0b110001010]=324;
angle[0b100001010]=325;
angle[0b101001010]=326;
angle[0b101101010]=327;
angle[0b111101010]=328;
angle[0b111001010]=329;
angle[0b111011010]=330;
angle[0b111011011]=331;
angle[0b101011011]=332;
angle[0b101011001]=333;
angle[0b100011001]=334;
angle[0b100111001]=335;
angle[0b100101001]=336;
angle[0b101101001]=337;
angle[0b111101001]=338;
angle[0b111100001]=339;
angle[0b111100011]=340;
angle[0b110100011]=341;
angle[0b110000011]=342;
angle[0b110001011]=343;
angle[0b110011011]=344;
angle[0b110011111]=345;
angle[0b110111111]=346;
angle[0b110101111]=347;
angle[0b100101111]=348;
angle[0b100101110]=349;
angle[0b000101110]=350;
angle[0b001101110]=351;
angle[0b001001110]=352;
angle[0b001001010]=353;
angle[0b001001011]=354;
angle[0b011001011]=355;
angle[0b011000011]=356;
angle[0b011000010]=357;
angle[0b010000010]=358;
angle[0b010000000]=359;
#endif
#ifdef BITS6
angle[0b000011]=0;
angle[0b010011]=1;
angle[0b110011]=2;
angle[0b110001]=3;
angle[0b110101]=4;
angle[0b100101]=5;
angle[0b000101]=6;
angle[0b000001]=7;
angle[0b100001]=8;
angle[0b101001]=9;
angle[0b111001]=10;
angle[0b111000]=11;
angle[0b111010]=12;
angle[0b110010]=13;
angle[0b100010]=14;
angle[0b100000]=15;
angle[0b110000]=16;
angle[0b110100]=17;
angle[0b111100]=18;
angle[0b011100]=19;
angle[0b011101]=20;
angle[0b011001]=21;
angle[0b010001]=22;
angle[0b010000]=23;
angle[0b011000]=24;
angle[0b011010]=25;
angle[0b011110]=26;
angle[0b001110]=27;
angle[0b101110]=28;
angle[0b101100]=29;
angle[0b101000]=30;
angle[0b001000]=31;
angle[0b001100]=32;
angle[0b001101]=33;
angle[0b001111]=34;
angle[0b000111]=35;
angle[0b010111]=36;
angle[0b010110]=37;
angle[0b010100]=38;
angle[0b000100]=39;
angle[0b000110]=40;
angle[0b100110]=41;
angle[0b100111]=42;
angle[0b100011]=43;
angle[0b101011]=44;
angle[0b001011]=45;
angle[0b001010]=46;
angle[0b000010]=47;

#endif
  // Angle-Decodings for 9-Bit Single-Track Gray-Code
  /*
angle[0b1]=0;
angle[0b101]=1;
angle[0b100000101]=2;
angle[0b110000101]=3;
angle[0b10000101]=4;
angle[0b10000111]=5;
angle[0b10000011]=6;
angle[0b11000011]=7;
angle[0b1000011]=8;
angle[0b1000001]=9;
angle[0b1100001]=10;
angle[0b100001]=11;
angle[0b110001]=12;
angle[0b1110001]=13;
angle[0b1010001]=14;
angle[0b11010001]=15;
angle[0b10010001]=16;
angle[0b10110001]=17;
angle[0b11110001]=18;
angle[0b11111001]=19;
angle[0b1111001]=20;
angle[0b1101001]=21;
angle[0b101101001]=22;
angle[0b111101001]=23;
angle[0b110101001]=24;
angle[0b110111001]=25;
angle[0b110111011]=26;
angle[0b110101011]=27;
angle[0b10101011]=28;
angle[0b11101011]=29;
angle[0b111101011]=30;
angle[0b101101011]=31;
angle[0b101101111]=32;
angle[0b111101111]=33;
angle[0b111001111]=34;
angle[0b110001111]=35;
angle[0b100001111]=36;
angle[0b1111]=37;
angle[0b111]=38;
angle[0b11]=39;
angle[0b10]=40;
angle[0b1010]=41;
angle[0b1011]=42;
angle[0b100001011]=43;
angle[0b100001010]=44;
angle[0b100001110]=45;
angle[0b100000110]=46;
angle[0b110000110]=47;
angle[0b10000110]=48;
angle[0b10000010]=49;
angle[0b11000010]=50;
angle[0b1000010]=51;
angle[0b1100010]=52;
angle[0b11100010]=53;
angle[0b10100010]=54;
angle[0b110100010]=55;
angle[0b100100010]=56;
angle[0b101100010]=57;
angle[0b111100010]=58;
angle[0b111110010]=59;
angle[0b11110010]=60;
angle[0b11010010]=61;
angle[0b11010011]=62;
angle[0b111010011]=63;
angle[0b101010011]=64;
angle[0b101110011]=65;
angle[0b101110111]=66;
angle[0b101010111]=67;
angle[0b101010110]=68;
angle[0b111010110]=69;
angle[0b111010111]=70;
angle[0b11010111]=71;
angle[0b11011111]=72;
angle[0b111011111]=73;
angle[0b110011111]=74;
angle[0b100011111]=75;
angle[0b11111]=76;
angle[0b11110]=77;
angle[0b1110]=78;
angle[0b110]=79;
angle[0b100]=80;
angle[0b10100]=81;
angle[0b10110]=82;
angle[0b10111]=83;
angle[0b10101]=84;
angle[0b11101]=85;
angle[0b1101]=86;
angle[0b100001101]=87;
angle[0b100001100]=88;
angle[0b100000100]=89;
angle[0b110000100]=90;
angle[0b10000100]=91;
angle[0b11000100]=92;
angle[0b111000100]=93;
angle[0b101000100]=94;
angle[0b101000101]=95;
angle[0b1000101]=96;
angle[0b11000101]=97;
angle[0b111000101]=98;
angle[0b111100101]=99;
angle[0b111100100]=100;
angle[0b110100100]=101;
angle[0b110100110]=102;
angle[0b110100111]=103;
angle[0b10100111]=104;
angle[0b11100111]=105;
angle[0b11101111]=106;
angle[0b10101111]=107;
angle[0b10101101]=108;
angle[0b110101101]=109;
angle[0b110101111]=110;
angle[0b110101110]=111;
angle[0b110111110]=112;
angle[0b110111111]=113;
angle[0b100111111]=114;
angle[0b111111]=115;
angle[0b111110]=116;
angle[0b111100]=117;
angle[0b11100]=118;
angle[0b1100]=119;
angle[0b1000]=120;
angle[0b101000]=121;
angle[0b101100]=122;
angle[0b101110]=123;
angle[0b101010]=124;
angle[0b111010]=125;
angle[0b11010]=126;
angle[0b11011]=127;
angle[0b11001]=128;
angle[0b1001]=129;
angle[0b100001001]=130;
angle[0b100001000]=131;
angle[0b110001000]=132;
angle[0b110001001]=133;
angle[0b10001001]=134;
angle[0b10001011]=135;
angle[0b10001010]=136;
angle[0b110001010]=137;
angle[0b110001011]=138;
angle[0b111001011]=139;
angle[0b111001001]=140;
angle[0b101001001]=141;
angle[0b101001101]=142;
angle[0b101001111]=143;
angle[0b101001110]=144;
angle[0b111001110]=145;
angle[0b111011110]=146;
angle[0b101011110]=147;
angle[0b101011010]=148;
angle[0b101011011]=149;
angle[0b101011111]=150;
angle[0b101011101]=151;
angle[0b101111101]=152;
angle[0b101111111]=153;
angle[0b1111111]=154;
angle[0b1111110]=155;
angle[0b1111100]=156;
angle[0b1111000]=157;
angle[0b111000]=158;
angle[0b11000]=159;
angle[0b10000]=160;
angle[0b1010000]=161;
angle[0b1011000]=162;
angle[0b1011100]=163;
angle[0b1010100]=164;
angle[0b1110100]=165;
angle[0b110100]=166;
angle[0b110110]=167;
angle[0b110010]=168;
angle[0b10010]=169;
angle[0b10011]=170;
angle[0b10001]=171;
angle[0b100010001]=172;
angle[0b100010011]=173;
angle[0b100010010]=174;
angle[0b100010110]=175;
angle[0b100010100]=176;
angle[0b100010101]=177;
angle[0b100010111]=178;
angle[0b110010111]=179;
angle[0b110010011]=180;
angle[0b10010011]=181;
angle[0b10011011]=182;
angle[0b10011111]=183;
angle[0b10011101]=184;
angle[0b110011101]=185;
angle[0b110111101]=186;
angle[0b10111101]=187;
angle[0b10110101]=188;
angle[0b10110111]=189;
angle[0b10111111]=190;
angle[0b10111011]=191;
angle[0b11111011]=192;
angle[0b11111111]=193;
angle[0b11111110]=194;
angle[0b11111100]=195;
angle[0b11111000]=196;
angle[0b11110000]=197;
angle[0b1110000]=198;
angle[0b110000]=199;
angle[0b100000]=200;
angle[0b10100000]=201;
angle[0b10110000]=202;
angle[0b10111000]=203;
angle[0b10101000]=204;
angle[0b11101000]=205;
angle[0b1101000]=206;
angle[0b1101100]=207;
angle[0b1100100]=208;
angle[0b100100]=209;
angle[0b100110]=210;
angle[0b100010]=211;
angle[0b100011]=212;
angle[0b100111]=213;
angle[0b100101]=214;
angle[0b101101]=215;
angle[0b101001]=216;
angle[0b101011]=217;
angle[0b101111]=218;
angle[0b100101111]=219;
angle[0b100100111]=220;
angle[0b100100110]=221;
angle[0b100110110]=222;
angle[0b100111110]=223;
angle[0b100111010]=224;
angle[0b100111011]=225;
angle[0b101111011]=226;
angle[0b101111010]=227;
angle[0b101101010]=228;
angle[0b101101110]=229;
angle[0b101111110]=230;
angle[0b101110110]=231;
angle[0b111110110]=232;
angle[0b111111110]=233;
angle[0b111111100]=234;
angle[0b111111000]=235;
angle[0b111110000]=236;
angle[0b111100000]=237;
angle[0b11100000]=238;
angle[0b1100000]=239;
angle[0b1000000]=240;
angle[0b101000000]=241;
angle[0b101100000]=242;
angle[0b101110000]=243;
angle[0b101010000]=244;
angle[0b111010000]=245;
angle[0b11010000]=246;
angle[0b11011000]=247;
angle[0b11001000]=248;
angle[0b1001000]=249;
angle[0b1001100]=250;
angle[0b1000100]=251;
angle[0b1000110]=252;
angle[0b1001110]=253;
angle[0b1001010]=254;
angle[0b1011010]=255;
angle[0b1010010]=256;
angle[0b1010110]=257;
angle[0b1011110]=258;
angle[0b1011111]=259;
angle[0b1001111]=260;
angle[0b1001101]=261;
angle[0b1101101]=262;
angle[0b1111101]=263;
angle[0b1110101]=264;
angle[0b1110111]=265;
angle[0b11110111]=266;
angle[0b11110101]=267;
angle[0b11010101]=268;
angle[0b11011101]=269;
angle[0b11111101]=270;
angle[0b11101101]=271;
angle[0b111101101]=272;
angle[0b111111101]=273;
angle[0b111111001]=274;
angle[0b111110001]=275;
angle[0b111100001]=276;
angle[0b111000001]=277;
angle[0b111000000]=278;
angle[0b11000000]=279;
angle[0b10000000]=280;
angle[0b10000001]=281;
angle[0b11000001]=282;
angle[0b11100001]=283;
angle[0b10100001]=284;
angle[0b110100001]=285;
angle[0b110100000]=286;
angle[0b110110000]=287;
angle[0b110010000]=288;
angle[0b10010000]=289;
angle[0b10011000]=290;
angle[0b10001000]=291;
angle[0b10001100]=292;
angle[0b10011100]=293;
angle[0b10010100]=294;
angle[0b10110100]=295;
angle[0b10100100]=296;
angle[0b10101100]=297;
angle[0b10111100]=298;
angle[0b10111110]=299;
angle[0b10011110]=300;
angle[0b10011010]=301;
angle[0b11011010]=302;
angle[0b11111010]=303;
angle[0b11101010]=304;
angle[0b11101110]=305;
angle[0b111101110]=306;
angle[0b111101010]=307;
angle[0b110101010]=308;
angle[0b110111010]=309;
angle[0b111111010]=310;
angle[0b111011010]=311;
angle[0b111011011]=312;
angle[0b111111011]=313;
angle[0b111110011]=314;
angle[0b111100011]=315;
angle[0b111000011]=316;
angle[0b110000011]=317;
angle[0b110000001]=318;
angle[0b110000000]=319;
angle[0b100000000]=320;
angle[0b100000010]=321;
angle[0b110000010]=322;
angle[0b111000010]=323;
angle[0b101000010]=324;
angle[0b101000011]=325;
angle[0b101000001]=326;
angle[0b101100001]=327;
angle[0b100100001]=328;
angle[0b100100000]=329;
angle[0b100110000]=330;
angle[0b100010000]=331;
angle[0b100011000]=332;
angle[0b100111000]=333;
angle[0b100101000]=334;
angle[0b101101000]=335;
angle[0b101001000]=336;
angle[0b101011000]=337;
angle[0b101111000]=338;
angle[0b101111100]=339;
angle[0b100111100]=340;
angle[0b100110100]=341;
angle[0b110110100]=342;
angle[0b111110100]=343;
angle[0b111010100]=344;
angle[0b111011100]=345;
angle[0b111011101]=346;
angle[0b111010101]=347;
angle[0b101010101]=348;
angle[0b101110101]=349;
angle[0b111110101]=350;
angle[0b110110101]=351;
angle[0b110110111]=352;
angle[0b111110111]=353;
angle[0b111100111]=354;
angle[0b111000111]=355;
angle[0b110000111]=356;
angle[0b100000111]=357;
angle[0b100000011]=358;
angle[0b100000001]=359;
*/
  Serial.begin(230400);
  for(i=2;i<=10;i++)
    pinMode(i,INPUT);
  Serial.println("9-Bit Single Track Gray Encoder");

#ifdef DISPLAY_THREE_DIGITS  
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  

  // initialize Timer1 for Multiplexing the 7-Segment Displays
  cli(); // Loeche globales Interruptflag
  TCNT1 = 0; //Timer Counter 1
  TCCR1A = 0; //Timer Counter Controll Register A
  TCCR1B = 0; //Timer Counter Controll Register B
  OCR1A = 312; //Output Compare Register A
  // Setze CS10 und CS12 - Clock Select Bit 10/12 (Prescaler 1024)
  TCCR1B |=  (1 << CS12);
  //CTC-Mode ein
  TCCR1B |= (1 << WGM12); // CTC-Mode (Clear Timer and Compare)
  //Timer/Counter Interrupt Mask Register
  TIMSK1 |= (1 << OCIE1A); //Output Compare A Match Interrupt Enable
  sei(); //Setze globales Interruptflag

//Mehr Infos: https://arduino-projekte.webnode.at/meine-projekte/servosteuerung/servotest/variante2/
#endif

}

#ifdef DISPLAY_THREE_DIGITS
/**
 * Multiplexer-Interrupt invoked by Timer
 * each time invoked next digit is dipslayed
 */
ISR(TIMER1_COMPA_vect) //Interrupt-Serviceroutine
{
   int e,z,h;
   digitalWrite(latchPin, LOW);
   h = display_value / 100;
   z = (display_value - h * 100) / 10;
   e = (display_value - h * 100 - z * 10);
   
   switch(digit)
   {
    case 0: shiftOut(dataPin, clockPin, MSBFIRST, h |  0b00010000); digit=1; break;
    case 1: shiftOut(dataPin, clockPin, MSBFIRST, z  | 0b00100000); digit=2; break;
    case 2: shiftOut(dataPin, clockPin, MSBFIRST, e  | 0b01000000); digit=0; break;
   }
    
    //return the latch pin high to signal chip that it
    //no longer needs to listen for information
    digitalWrite(latchPin, HIGH); 
}
#endif

void printBinary(int in)
{
  int s;
  s=256; //1<<8;
  for (int b = 8; b >= 0; b--)
  {
    if(s & in)
      Serial.print(1);
    else
      Serial.print(0);
      s /= 2;
  }
}

int changedBits(int a, int b)
{
  int i,n=0;
  for(i=0;i<9;i++) 
     if( (a & (1<<i)) != (b & (1<<i))) n++;
  return n; 
}

 int reverse_Bits(int n, int no_of_bits){
    int result = 0;
    for(int i=0;i<no_of_bits;i++){
        result <<= 1;
        result |= n & 1;
        n >>= 1;
    }
    return result;
 }
 
volatile int bold=0;
void loop() {
   int b;
   cli();
   b = (PIND >> 2) | ((PINB & 0b00000111) << 6);
#ifdef BITS6
   b = b & 0b00111111;
#endif

//  b=reverse_Bits(b,6);
   
 //  printBinary(b); Serial.println();
   sei();

   if(b!=bold)
   {
#ifdef DISPLAY_THREE_DIGITS
      display_value = angle[b];
      noInterrupts();
#endif
      Serial.print( angle[b]);
      Serial.print(" ");
      printBinary(b);
      Serial.print(" raw=");
      Serial.print(b,HEX);
      Serial.print(" -> ");
      Serial.print(angle[b],HEX);
      Serial.print(" ");
      Serial.print( changedBits(b,bold));
      Serial.println();
#ifdef DISPLAY_THREE_DIGITS
      interrupts();
#endif
   }
   bold=b;
   delay(50);
   
}
